#!/bin/sh
#
# 2010/01/02
# Federico A. Bareilles
# 


NO_ARGS=0
E_OPTERROR=65

get_blobid()
{
    grep "$1" $2 |sed 's/.*blobid="\([0-9]\+\)".*/\1/'
}

get_depth()
{
    grep "$1" $2 |sed 's/.*depth="\([0-9]\+\)".*/\1/'
}

get_max()
{
    grep "$1" $2 |sed 's/.*max="\([0-9.\-]\+\)".*/\1/'
}

get_min()
{
    grep "$1" $2 |sed 's/.*min="\([0-9.\-]\+\)".*/\1/'
}

get_val()
{
    grep "$1" $2 |sed 's/[[:space:]]*<.*>\([0-9.\-]\+\)<\/.*>/\1/'
}

get_type()
{
    grep "$1" $2 |sed 's/.*type="\([KDPdBZ]\+\)".*/\1/'|\
                  tr '[[:upper:]]' '[[:lower:]]'
}

dump_blob_id()
{
    FILE=$1
    export BLID=$2
    cat $FILE |\
    awk 'BEGIN{dump=0; id=ENVIRON["BLID"];} /<\/BLOB>/{dump=0} {if(dump) print $0;} /<BLOB blobid=\"([0-9]+)\".*>/{split($0,num,/\"/); if(num[2] == id) dump = 1;}'
}


OUTRAYINFO=false

if [ $# -eq "$NO_ARGS" ]  # Script invoked with no command-line args?
then
  echo "Usage: `basename $0` -s slice_id [-a] file_name" >&2
  exit $E_OPTERROR
fi

while getopts "s:a" Option
do
  case $Option in
    s     ) SLICE=$OPTARG;;
    a     ) OUTRAYINFO=true;;
    *     ) echo "Unimplemented option chosen." >&2
	  exit $E_OPTERROR
	  ;;
  esac
done

shift $(($OPTIND - 1))


DATAFILE=$1

if [ ! -f $DATAFILE ]; then
    echo "$DATAFILE not found." >&2
    exit 1
fi

export SLICE

TMPFILE=/tmp/blobdump-$(date +%s).tmp

cat $DATAFILE |\
awk 'BEGIN{dump=0; slice=ENVIRON["SLICE"];} /<\/slice>/{dump=0} {if(dump) print $0;} /<slice refid=\"([0-9]+)\".*>/{split($0,num,/\"/); if(num[2] == slice) dump = 1;}'|\
grep -E '(rayinfo|rawdata|posangle)' > $TMPFILE

RAYINFO_BLID=$(get_blobid rayinfo $TMPFILE)
RAYINFO_DEPTH=$(get_depth rayinfo $TMPFILE)

RAWDATA_BLID=$(get_blobid rawdata $TMPFILE)
RAWDATA_DEPTH=$(get_depth rawdata $TMPFILE)
RAWDATA_MIN=$(get_min rawdata $TMPFILE)
RAWDATA_MAX=$(get_max rawdata $TMPFILE)
RAWDATA_TYPE=$(get_type rawdata $TMPFILE)

POSANGLE=$(get_val posangle $TMPFILE)

rm -f $TMPFILE

if [ "x$OUTRAYINFO" = "xfalse" ]; then
    dump_blob_id $DATAFILE $RAWDATA_BLID |\
    blob2txt -v --$RAWDATA_TYPE --emin $RAWDATA_MIN --emax $RAWDATA_MAX 
else
    dump_blob_id $DATAFILE $RAYINFO_BLID |\
    blob2txt -v --rayinfo 
fi