/** 
 *   geo: Calculate distance, bearing and more between
 *   Latitude/Longitude points
 *
 *   Based on: http://www.movable-type.co.uk/scripts/latlong.html
 *
 *   (c) Copyright 2009 Federico Bareilles <bareilles@gmail.com>,
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/
#define _GNU_SOURCE

#include <math.h>

#define R_EART 6371.0 //Km


double geo_distance(double from_lat, double from_lon,
		       double to_lat, double to_lon )
{
	double delta_lat = to_lat - from_lat;
	double delta_lon = to_lon - from_lon;
	double a = pow(sin(delta_lat/2.0),2.0) + cos(from_lat) * cos(to_lat) * \
		pow(sin(delta_lon/2.0),2.0) ;
	
	return 2.0 * atan2(sqrt(a), sqrt(1.0 - a)) * (R_EART);
}


double geo_bearing(double from_lat, double from_lon,
	       double to_lat, double to_lon )
{
	double theta = atan2(sin(to_lon - from_lon) * cos(to_lat),
			     cos(from_lat) * sin(to_lat) -
			     sin(from_lat) * cos(to_lat) * 
			     cos(to_lon - from_lon));
			   
	return theta;
}



void geo_dist_ber2dest_point(double from_lat, double from_lon,
			     double theta, double distance,
			     double *to_lat, double *to_lon)
{	
	*to_lat = asin(sin(from_lat) * cos(distance/R_EART) + 
		       cos(from_lat) * sin(distance/R_EART) * cos(theta));
	*to_lon = from_lon + 
		atan2(sin(theta) * sin(distance/R_EART) * cos(from_lat), 
		      cos(distance/R_EART) - sin(from_lat) * sin(*to_lat));
	
	return;
}
