/** 
 *   my_xcomp: Rapper para las funciones de des/compresion sobre zlib
 *
 *   (c) Copyright 2009 Federico Bareilles <bareilles@gmail.com>,
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/
#define _GNU_SOURCE

#include <stdlib.h>
#include <stdint.h>
#include <zlib.h>
#include <unistd.h>
#include <my_stdio.h>

#define MY_MAX(a, b) ((b) < (a) ? (a) : (b))


void *my_comp( const void *data_in, size_t  size_in, size_t  *size_out)
{
	void *data_out = NULL;
	unsigned long len = size_in + size_in / 100 + 13;
	int ret;

	if ( size_in == 0 ) {
		return data_out;
	}
	if ( data_in == NULL) {
		PVERB(PWARN, "data_in is NULL." );
		return data_out;
	}
	if ( (data_out = (void *) malloc(len + 4)) == NULL ) {
		PVERB(PERR,"Not enough memory.");
		return NULL;
	}
	do {
		*size_out = len;
		ret = compress((unsigned char *) data_out + 4, &len, 
			       (unsigned char *) data_in, size_in);
		
		switch ( ret ) {
		case Z_BUF_ERROR:
			len *= 2;
		case Z_OK:
			if ( *size_out != len) {
				if ((data_out = 
				     realloc (data_out, len + 4)) == NULL)
					return NULL;
				*((unsigned char *) data_out) = 
					( size_in & 0xff000000 ) >> 24;
				*((unsigned char *) data_out+1) = 
					( size_in & 0x00ff0000 ) >> 16;
				*((unsigned char *) data_out+2) = 
					( size_in & 0x0000ff00 ) >> 8;
				*((unsigned char *) data_out+3) = 
					( size_in & 0x000000ff );
			}
			break;
		case Z_MEM_ERROR:
			PVERB(PERR, "Z_MEM_ERROR: Not enough memory." );
			break;
			
		}
	} while ( ret == Z_BUF_ERROR );
	
	if ( ret != Z_OK ) {
		free(data_out);
		data_out = NULL;
		*size_out = 0;
	} else {
		*size_out = len + 4;
	}

	return data_out;
}


void *my_uncomp( const void *data_in, size_t  size_in, size_t  *size_out)
{
	void *data_out = NULL;
	unsigned char *data = (unsigned char*) data_in;
	int ret;
	unsigned long expected_size = (data[0] << 24)|(data[1] << 16)|
		(data[2] <<  8)|data[3];
	unsigned long len = MY_MAX( expected_size, 1);
    
	if ( size_in <= 4 ) {
		if (size_in < 4 || 
		    (data[0]!=0 || data[1]!=0 || data[2]!=0 || data[3]!=0) )
			PVERB(PWARN, "Input data is corrupted." );
		return NULL;
	}
	if ( (data_out = (void *) malloc(len)) == NULL ) {
		PVERB(PERR,"Not enough memory.");
		return NULL;
	}

	do {
		*size_out = len;
		ret = uncompress( (unsigned char *) data_out, &len,
				  data+4, size_in - 4 );
		switch ( ret ) {
		case Z_MEM_ERROR:
			PVERB(PERR, "Z_MEM_ERROR: Not enough memory." );
			break;
		case Z_BUF_ERROR:
			len *= 2;
		case Z_OK:
			if ( len != *size_out ) {
				PVERB(PERR,"len != *size_out? Why??? (ret: %d",
				      ret);
				PVERB(PERR,"len: %ld, *size_out: %d",
				      len, *size_out);
				if ((data_out = realloc (data_out, len)) == NULL)
					return NULL;
				//*size_out = len;
			}
			break;
		case Z_DATA_ERROR:
			PVERB(PERR, "Z_DATA_ERROR: Input data is corrupted." );
			break;
		}
	} while ( ret == Z_BUF_ERROR );

	if ( ret != Z_OK ) {
		free(data_out);
		data_out = NULL;
		*size_out = 0;
	} else {
		*size_out = len;
	}

	return data_out;
}
