#!/bin/sh
#
# 2010/01/05
# Federico A. Bareilles
# 


NO_ARGS=0
E_OPTERROR=65

get_blobid()
{
    grep "$1" $2 |sed 's/.*blobid="\([0-9]\+\)".*/\1/'
}

get_depth()
{
    grep "$1" $2 |sed 's/.*depth="\([0-9]\+\)".*/\1/'
}

get_max()
{
    grep "$1" $2 |sed 's/.*max="\([0-9.\-]\+\)".*/\1/'
}

get_min()
{
    grep "$1" $2 |sed 's/.*min="\([0-9.\-]\+\)".*/\1/'
}

get_val()
{
    grep "$1" $2 |sed 's/[[:space:]]*<.*>\([0-9.\-]\+\)<\/.*>/\1/'
}

get_type()
{
    grep "$1" $2 |sed 's/.*type="\([KDPdBZ]\+\)".*/\1/'|\
                  tr '[[:upper:]]' '[[:lower:]]'
}

dump_blob_id()
{
    FILE=$1
    export BLID=$2
    cat $FILE |\
    awk 'BEGIN{dump=0; id=ENVIRON["BLID"];} /<\/BLOB>/{dump=0} {if(dump) print $0;} /<BLOB blobid=\"([0-9]+)\".*>/{split($0,num,/\"/); if(num[2] == id) dump = 1;}'
}

get_header(){
    awk 'BEGIN{out=1;} /<!-- END XML -->/{out=0}{if(out)print $0}' $@
}


if [ $# -eq "$NO_ARGS" ]  # Script invoked with no command-line args?
then
  echo "Usage: `basename $0` -s slice_id [-a] file_name" >&2
  exit $E_OPTERROR
fi

while getopts "s:a" Option
do
  case $Option in
    s     ) SLICE=$OPTARG;;
    *     ) echo "Unimplemented option chosen." >&2
	  exit $E_OPTERROR
	  ;;
  esac
done

shift $(($OPTIND - 1))


DATAFILE=$1

if [ ! -f $DATAFILE ]; then
    echo "$DATAFILE not found." >&2
    exit 1
fi

NUMELE=$(get_header $DATAFILE|grep -F numele|\
         sed 's/[[:space:]]*<.*>\([0-9.\-]\+\)<\/.*>/\1/')
BEAMWIDTH=$(get_header $DATAFILE|grep -F beamwidth|\
         sed 's/[[:space:]]*<.*>\([0-9.\-]\+\)<\/.*>/\1/')

export BEAMWIDTH
#SLICE_LIST=$(get_header $DATAFILE|\
#             grep -F '<slice refid='|sed 's/.*refid="\([0-9]\+\)".*/\1/')

NUMELE=1
for SLICE in $(seq 0 $(($NUMELE-1))); do
    export SLICE
    TMPFILE=/tmp/vol2map-$(date +%s)-$SLICE.tmp
    get_header $DATAFILE|\
    awk 'BEGIN{dump=0; slice=ENVIRON["SLICE"];} 
        /<\/slice>/{dump=0} {if(dump) print $0;} 
        /<slice refid=\"([0-9]+)\".*>/{
          split($0,num,/\"/);if(num[2] == slice) dump = 1;
        }'|\
        grep -E '(rayinfo|rawdata|posangle)' > $TMPFILE

    RAYINFO_BLID=$(get_blobid rayinfo $TMPFILE)
    RAYINFO_DEPTH=$(get_depth rayinfo $TMPFILE)

    RAWDATA_BLID=$(get_blobid rawdata $TMPFILE)
    RAWDATA_DEPTH=$(get_depth rawdata $TMPFILE)
    RAWDATA_MIN=$(get_min rawdata $TMPFILE)
    RAWDATA_MAX=$(get_max rawdata $TMPFILE)
    RAWDATA_TYPE=$(get_type rawdata $TMPFILE)

    POSANGLE=$(get_val posangle $TMPFILE)
    
    rm -f $TMPFILE

    #echo $SLICE $RAYINFO_BLID $RAWDATA_BLID

    TMPRAY=/tmp/vol2map-$(date +%s)-$SLICE-ray.tmp
    export POSANGLE TMPRAY TMPRAW
    dump_blob_id $DATAFILE $RAYINFO_BLID |blob2txt  --rayinfo |\
    awk 'BEGIN{pi=4.0*atan2(1.0,1.0); fa=pi/180.0; 
               cosele=cos(ENVIRON["POSANGLE"]*fa);}
         {print cos($1*fa)*cosele,sin($1*fa)*cosele;}' > $TMPRAY
    dump_blob_id $DATAFILE $RAWDATA_BLID |\
    blob2txt -v --$RAWDATA_TYPE --emin $RAWDATA_MIN --emax $RAWDATA_MAX |\
    awk 'function abs(x){if(x<0.0) return -x;else return x;};
         function round(x){iround=int(x); if((x-iround)>.5) iround++;return iround};
         BEGIN{while(getline <ENVIRON["TMPRAY"]){
                  xfa[i]=$1;yfa[i]=$2;i++;
               };
               beamw2=ENVIRON["BEAMWIDTH"]*atan2(1.0,1.0)/45.0/2.0;
               sinbw2=sin(beamw2);
               sinbw2=0.0;
               dim=480;
               scale=2.0;
               }
         {l=(NR-1)%dim;
           if ($1 != -100) {
             x=((l*xfa[an]+dim)/scale);
             y=((l*yfa[an]+dim)/scale);
             widthx=abs(l*sinbw2*yfa[an]);
             if(widthx == 0) widthx++;
             widthy=abs(l*sinbw2*xfa[an]);
             if(widthy == 0) widthy++;
             if(widthx>=widthy){
                for(i=-widthx*2; i<=widthx*2; i++){
                   j=(xfa[an]/yfa[an] * (-i/2));
                   rx=round(x+i/2);
                   ry=round(y+j);
                   v[ry,rx]+=$1;
                   count[ry,rx]++;
                }
             } else { 
                for(j=-widthy*2; j<=widthy*2; j++){
                   i=(yfa[an]/xfa[an] * (-j/2));
                   rx=round(x+i);
                   ry=round(y+j/2);
                   v[ry,rx]+=$1;
                   count[ry,rx]++;
                }
             }
          }
          if(l==(dim-1))an++;
          }
          END{ for(x=0;x<(dim*2/scale);x++){
                  for(y=0;y<(dim*2/scale);y++){
                     if(count[y,x] > 0) print v[y,x]/count[y,x];
                     else print 0;
                  }
              }
          }'
    #>$(basename $DATAFILE .vol)-$POSANGLE.dat

    rm -f $TMPRAY
done

exit 1
if [ "x$OUTRAYINFO" = "xfalse" ]; then
    dump_blob_id $DATAFILE $RAWDATA_BLID |\
    blob2txt -v --$RAWDATA_TYPE --emin $RAWDATA_MIN --emax $RAWDATA_MAX 
else
    dump_blob_id $DATAFILE $RAYINFO_BLID |\
    blob2txt -v --rayinfo 
fi
